@@ -1,11 +1,10 @@
 /* lsmbox -- list nr of total and read messages for one or several mailboxes
  *
- * Copyright (C) 2002-2006 David Weinehall
+ * Copyright (C) 2002-2006, 2010 David Weinehall
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +12,9 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software Foundation,
- *  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ *  along with this program; if not, write to
+ *  the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 #include "lsmbox.h"
 
@@ -323,6 +323,27 @@ char *basenamep(const char *const str)
 	return tmp;
 }
 
+/**
+ * Move to previous existing mailbox
+ */
+static void previous_mbox(void)
+{
+	while (selected > -1) {
+		if (mboxes[--selected].exists)
+			break;
+	}
+}
+
+/**
+ * Move to next existing mailbox
+ */
+static void next_mbox(void)
+{
+	while (selected == -1 || mboxes[selected].name)
+		if (mboxes[++selected].exists)
+			break;
+}
+
 /**
  * Check for keypresses and mouse-events
  *
@@ -342,16 +363,12 @@ static int eventcheck(void)
 			break;
 
 		case KEY_UP:
-			if (selected > 0)
-				selected--;
-			else if (selected == -1)
-				selected = INT_MAX;
-
+			previous_mbox();
 			selectedtimer = 0;
 			break;
 
 		case KEY_DOWN:
-			selected++;
+			next_mbox();
 			selectedtimer = 0;
 			break;
 
@@ -1389,6 +1406,9 @@ static int update_mailbox_stats(void)
 
 		if (stat(name, &st) == -1) {
 			if (errno == ENOENT) {
+				if (mboxes[i].exists && i == selected)
+					previous_mbox();
+
 				mboxes[i].exists = 0;
 				errno = 0;
 				continue;
@@ -1905,6 +1925,7 @@ int main(int argc, char *argv[])
 		/* Check for a keypress every second */
 		if (!quit)
 			(void)sleep((unsigned int)1);
+
 		if (errno == EINTR)
 			errno = 0;
 	} while (!errno && continuous && !quit);
